@@ -11,28 +11,7 @@ use crate::consts;
 use crate::error::{MainError, MainResult};
 use crate::templates;
 use crate::Input;
-use lazy_static::lazy_static;
 use log::{error, info};
-use std::ffi::OsString;
-
-lazy_static! {
-    static ref RE_SHORT_MANIFEST: Regex =
-        Regex::new(r"^(?i)\s*//\s*cargo-deps\s*:(.*?)(\r\n|\n)").unwrap();
-    static ref RE_MARGIN: Regex = Regex::new(r"^\s*\*( |$)").unwrap();
-    static ref RE_SPACE: Regex = Regex::new(r"^(\s+)").unwrap();
-    static ref RE_NESTING: Regex = Regex::new(r"/\*|\*/").unwrap();
-    static ref RE_COMMENT: Regex = Regex::new(r"^\s*//(!|/)").unwrap();
-    static ref RE_SHEBANG: Regex = Regex::new(r"^#![^\[].*?(\r\n|\n)").unwrap();
-    static ref RE_CRATE_COMMENT: Regex = {
-        Regex::new(
-            r"(?x)
-                # We need to find the first `/*!` or `//!` that *isn't* preceeded by something that would make it apply to anything other than the crate itself.  Because we can't do this accurately, we'll just require that the doc comment is the *first* thing in the file (after the optional shebang, which should already have been stripped).
-                ^\s*
-                (/\*!|//(!|/))
-            "
-        ).unwrap()
-    };
-}
 
 /**
 Splits input into a complete Cargo manifest and unadultered Rust source.
@@ -43,46 +22,50 @@ pub fn split_input(
     input: &Input,
     deps: &[(String, String)],
     prelude_items: &[String],
-    input_id: &OsString,
+    bin_name: &str,
+    script_name: &str,
+    toolchain: Option<String>,
 ) -> MainResult<(String, String)> {
-    fn contains_main_method(line: &str) -> bool {
-        let line = line.trim_start();
-        line.starts_with("fn main(")
-            || line.starts_with("pub fn main(")
-            || line.starts_with("async fn main(")
-            || line.starts_with("pub async fn main(")
+    fn contains_main_method(source: &str) -> bool {
+        let re_shebang: Regex =
+            Regex::new(r#"(?m)^ *(pub )?(async )?(extern "C" )?fn main *\("#).unwrap();
+        re_shebang.is_match(source)
     }
 
-    let template_buf;
-    let (part_mani, source, template, sub_prelude) = match *input {
-        Input::File(_, _, content, _) => {
+    let (part_mani, source, template, sub_prelude) = match input {
+        Input::File(_, _, content) => {
             assert_eq!(prelude_items.len(), 0);
-            let content = strip_shebang(content);
+            let (content, shebang_used) = strip_shebang(content);
             let (manifest, source) =
                 find_embedded_manifest(content).unwrap_or((Manifest::Toml(""), content));
 
-            let source = if source.lines().any(contains_main_method) {
-                source.to_string()
+            let source = if contains_main_method(source) {
+                if shebang_used {
+                    format!("//\n{}", source)
+                } else {
+                    source.to_string()
+                }
             } else {
-                format!("fn main() -> Result<(), Box<dyn std::error::Error+Sync+Send>> {{\n    {{\n    {}    }}\n    Ok(())\n}}", source)
+                format!("fn main() -> Result<(), Box<dyn std::error::Error+Sync+Send>> {{    {{\n{}    }}\n    Ok(())\n}}", source)
             };
-            (manifest, source, templates::get_template("file")?, false)
-        }
-        Input::Expr(content, template) => {
-            template_buf = templates::get_template(template.unwrap_or("expr"))?;
-            let (manifest, template_src) = find_embedded_manifest(&template_buf)
-                .unwrap_or((Manifest::Toml(""), &template_buf));
-            (manifest, content.to_string(), template_src.into(), true)
-        }
-        Input::Loop(content, count) => {
-            let templ = if count { "loop-count" } else { "loop" };
-            (
-                Manifest::Toml(""),
-                content.to_string(),
-                templates::get_template(templ)?,
-                true,
-            )
+            (manifest, source, consts::FILE_TEMPLATE, false)
         }
+        Input::Expr(content) => (
+            Manifest::Toml(""),
+            content.to_string(),
+            consts::EXPR_TEMPLATE,
+            true,
+        ),
+        Input::Loop(content, count) => (
+            Manifest::Toml(""),
+            content.to_string(),
+            if *count {
+                consts::LOOP_COUNT_TEMPLATE
+            } else {
+                consts::LOOP_TEMPLATE
+            },
+            true,
+        ),
     };
 
     let mut prelude_str;
@@ -100,16 +83,13 @@ pub fn split_input(
         subs.insert(consts::SCRIPT_PRELUDE_SUB, &prelude_str[..]);
     }
 
-    let source = templates::expand(&template, &subs)?;
-
+    let source = templates::expand(template, &subs)?;
+    let part_mani = part_mani.into_toml()?;
     info!("part_mani: {:?}", part_mani);
     info!("source: {:?}", source);
 
-    let part_mani = part_mani.into_toml()?;
-    info!("part_mani: {:?}", part_mani);
-
     // It's-a mergin' time!
-    let def_mani = default_manifest(input, input_id)?;
+    let def_mani = default_manifest(input, bin_name, script_name, toolchain);
     let dep_mani = deps_manifest(deps)?;
 
     let mani = merge_manifest(def_mani, part_mani)?;
@@ -117,36 +97,35 @@ pub fn split_input(
 
     // Fix up relative paths.
     let mani = fix_manifest_paths(mani, &input.base_path())?;
-    info!("mani: {:?}", mani);
 
-    let mani_str = format!("{}", toml::Value::Table(mani));
-    info!("mani_str: {}", mani_str);
+    let mani_str = format!("{}", mani);
+    info!("manifest: {}", mani_str);
 
     Ok((mani_str, source))
 }
 
-#[rustversion::before(1.59)]
-#[cfg(test)]
-pub const STRIP_SECTION: &str = "\n";
-#[rustversion::since(1.59)]
 #[cfg(test)]
 pub const STRIP_SECTION: &str = r##"
+
 [profile.release]
 strip = true
 "##;
 
 #[test]
 fn test_split_input() {
-    let input_id = OsString::from("input_id");
+    let bin_name = "binary-name".to_string();
+    let script_name = "main.rs".to_string();
+    let toolchain = None;
     macro_rules! si {
         ($i:expr) => {
-            split_input(&$i, &[], &[], &input_id).ok()
+            split_input(&$i, &[], &[], &bin_name, &script_name, toolchain.clone()).ok()
         };
     }
 
-    let dummy_path: ::std::path::PathBuf = "p".into();
-    let dummy_path = &dummy_path;
-    let f = |c| Input::File("n", dummy_path, c, 0);
+    let f = |c: &str| {
+        let dummy_path: ::std::path::PathBuf = "main.rs".into();
+        Input::File("n".to_string(), dummy_path, c.to_string())
+    };
 
     macro_rules! r {
         ($m:expr, $r:expr) => {
@@ -160,8 +139,8 @@ fn test_split_input() {
             format!(
                 "{}{}",
                 r#"[[bin]]
-name = "n_input_id"
-path = "n.rs"
+name = "binary-name"
+path = "main.rs"
 
 [dependencies]
 
@@ -176,6 +155,87 @@ version = "0.1.0""#,
         )
     );
 
+    assert_eq!(
+        si!(f(r#"#!/usr/bin/env rust-script
+fn main() {}"#)),
+        r!(
+            format!(
+                "{}{}",
+                r#"[[bin]]
+name = "binary-name"
+path = "main.rs"
+
+[dependencies]
+
+[package]
+authors = ["Anonymous"]
+edition = "2021"
+name = "n"
+version = "0.1.0""#,
+                STRIP_SECTION
+            ),
+            r#"//
+fn main() {}"#
+        )
+    );
+
+    assert_eq!(
+        si!(f(r#"#[thingy]
+fn main() {}"#)),
+        r!(
+            format!(
+                "{}{}",
+                r#"[[bin]]
+name = "binary-name"
+path = "main.rs"
+
+[dependencies]
+
+[package]
+authors = ["Anonymous"]
+edition = "2021"
+name = "n"
+version = "0.1.0""#,
+                STRIP_SECTION
+            ),
+            r#"#[thingy]
+fn main() {}"#
+        )
+    );
+
+    assert_eq!(
+        split_input(
+            &f(r#"fn main() {}"#),
+            &[],
+            &[],
+            &bin_name,
+            "main.rs",
+            Some("stable".to_string())
+        )
+        .ok(),
+        r!(
+            format!(
+                "{}{}",
+                r#"[[bin]]
+name = "binary-name"
+path = "main.rs"
+
+[dependencies]
+
+[package]
+authors = ["Anonymous"]
+edition = "2021"
+name = "n"
+version = "0.1.0"
+
+[package.metadata.rustscript]
+toolchain = "stable""#,
+                STRIP_SECTION
+            ),
+            r#"fn main() {}"#
+        )
+    );
+
     // Ensure removed prefix manifests don't work.
     assert_eq!(
         si!(f(r#"
@@ -186,8 +246,8 @@ fn main() {}
             format!(
                 "{}{}",
                 r#"[[bin]]
-name = "n_input_id"
-path = "n.rs"
+name = "binary-name"
+path = "main.rs"
 
 [dependencies]
 
@@ -215,8 +275,8 @@ fn main() {}
             format!(
                 "{}{}",
                 r#"[[bin]]
-name = "n_input_id"
-path = "n.rs"
+name = "binary-name"
+path = "main.rs"
 
 [dependencies]
 
@@ -244,8 +304,8 @@ fn main() {}
             format!(
                 "{}{}",
                 r#"[[bin]]
-name = "n_input_id"
-path = "n.rs"
+name = "binary-name"
+path = "main.rs"
 
 [dependencies]
 time = "0.1.25"
@@ -273,8 +333,8 @@ fn main() {}
             format!(
                 "{}{}",
                 r#"[[bin]]
-name = "n_input_id"
-path = "n.rs"
+name = "binary-name"
+path = "main.rs"
 
 [dependencies]
 libc = "0.2.5"
@@ -310,8 +370,8 @@ fn main() {}
             format!(
                 "{}{}",
                 r#"[[bin]]
-name = "n_input_id"
-path = "n.rs"
+name = "binary-name"
+path = "main.rs"
 
 [dependencies]
 time = "0.1.25"
@@ -341,44 +401,14 @@ fn main() {}
 /**
 Returns a slice of the input string with the leading shebang, if there is one, omitted.
 */
-fn strip_shebang(s: &str) -> &str {
-    match RE_SHEBANG.find(s) {
-        Some(m) => &s[m.end()..],
-        None => s,
+fn strip_shebang(s: &str) -> (&str, bool) {
+    let re_shebang: Regex = Regex::new(r"^#![^\[].*?(\r\n|\n)").unwrap();
+    match re_shebang.find(s) {
+        Some(m) => (&s[m.end()..], true),
+        None => (s, false),
     }
 }
 
-#[test]
-fn test_strip_shebang() {
-    assert_eq!(
-        strip_shebang(
-            "\
-#!/usr/bin/env rust-script
-and the rest
-\
-        "
-        ),
-        "\
-and the rest
-\
-        "
-    );
-    assert_eq!(
-        strip_shebang(
-            "\
-#![thingy]
-and the rest
-\
-        "
-        ),
-        "\
-#![thingy]
-and the rest
-\
-        "
-    );
-}
-
 /**
 Represents the kind, and content of, an embedded manifest.
 */
@@ -648,10 +678,10 @@ fn main() {}
 Locates a "short comment manifest" in Rust source.
 */
 fn find_short_comment_manifest(s: &str) -> Option<(Manifest, &str)> {
+    let re: Regex = Regex::new(r"^(?i)\s*//\s*cargo-deps\s*:(.*?)(\r\n|\n)").unwrap();
     /*
     This is pretty simple: the only valid syntax for this is for the first, non-blank line to contain a single-line comment whose first token is `cargo-deps:`.  That's it.
     */
-    let re = &*RE_SHORT_MANIFEST;
     if let Some(cap) = re.captures(s) {
         if let Some(m) = cap.get(1) {
             return Some((Manifest::DepList(m.as_str()), s));
@@ -664,6 +694,15 @@ fn find_short_comment_manifest(s: &str) -> Option<(Manifest, &str)> {
 Locates a "code block manifest" in Rust source.
 */
 fn find_code_block_manifest(s: &str) -> Option<(Manifest, &str)> {
+    let re_crate_comment: Regex = {
+        Regex::new(
+            r"(?x)
+                # We need to find the first `/*!` or `//!` that *isn't* preceeded by something that would make it apply to anything other than the crate itself.  Because we can't do this accurately, we'll just require that the doc comment is the *first* thing in the file (after the optional shebang, which should already have been stripped).
+                ^\s*
+                (/\*!|//(!|/))
+            "
+        ).unwrap()
+    };
     /*
     This has to happen in a few steps.
 
@@ -673,7 +712,7 @@ fn find_code_block_manifest(s: &str) -> Option<(Manifest, &str)> {
 
     Then, we need to take the contents of this doc comment and feed it to a Markdown parser.  We are looking for *the first* fenced code block with a language token of `cargo`.  This is extracted and pasted back together into the manifest.
     */
-    let start = match RE_CRATE_COMMENT.captures(s) {
+    let start = match re_crate_comment.captures(s) {
         Some(cap) => match cap.get(1) {
             Some(m) => m.start(),
             None => return None,
@@ -849,9 +888,9 @@ fn extract_comment(s: &str) -> MainResult<String> {
         */
         let mut r = String::new();
 
-        let margin_re = &*RE_MARGIN;
-        let space_re = &*RE_SPACE;
-        let nesting_re = &*RE_NESTING;
+        let margin_re: Regex = Regex::new(r"^\s*\*( |$)").unwrap();
+        let space_re: Regex = Regex::new(r"^(\s+)").unwrap();
+        let nesting_re: Regex = Regex::new(r"/\*|\*/").unwrap();
 
         let mut leading_space = None;
         let mut margin = None;
@@ -923,8 +962,9 @@ fn extract_comment(s: &str) -> MainResult<String> {
     fn extract_line(s: &str) -> MainResult<String> {
         let mut r = String::new();
 
-        let comment_re = &*RE_COMMENT;
-        let space_re = &*RE_SPACE;
+        let comment_re = Regex::new(r"^\s*//(!|/)").unwrap();
+
+        let space_re = Regex::new(r"^(\s+)").unwrap();
 
         let mut leading_space = None;
 
@@ -1053,22 +1093,77 @@ time = "*"
 /**
 Generates a default Cargo manifest for the given input.
 */
-fn default_manifest(input: &Input, input_id: &OsString) -> MainResult<toml::value::Table> {
-    let mani_str = {
-        let pkg_name = input.package_name();
-        let bin_name = format!("{}_{}", &*pkg_name, input_id.to_str().unwrap());
-        let mut subs = HashMap::with_capacity(3);
-        subs.insert(consts::MANI_NAME_SUB, &*pkg_name);
-        subs.insert(consts::MANI_BIN_NAME_SUB, &*bin_name);
-        subs.insert(consts::MANI_FILE_SUB, input.safe_name());
-        templates::expand(consts::DEFAULT_MANIFEST, &subs)?
-    };
-    toml::from_str(&mani_str).map_err(|e| {
-        MainError::Tag(
-            "could not parse default manifest".into(),
-            Box::new(MainError::Other(Box::new(e))),
-        )
-    })
+fn default_manifest(
+    input: &Input,
+    bin_name: &str,
+    script_name: &str,
+    toolchain: Option<String>,
+) -> toml::value::Table {
+    let mut package_map = toml::map::Map::new();
+    package_map.insert(
+        "name".to_string(),
+        toml::value::Value::String(input.package_name()),
+    );
+    package_map.insert(
+        "version".to_string(),
+        toml::value::Value::String("0.1.0".to_string()),
+    );
+    package_map.insert(
+        "authors".to_string(),
+        toml::value::Value::Array(vec![toml::value::Value::String("Anonymous".to_string())]),
+    );
+    package_map.insert(
+        "edition".to_string(),
+        toml::value::Value::String("2021".to_string()),
+    );
+    if let Some(toolchain) = toolchain {
+        let mut metadata = toml::map::Map::new();
+        let mut rustscript_metadata = toml::map::Map::new();
+        rustscript_metadata.insert(
+            "toolchain".to_string(),
+            toml::value::Value::String(toolchain),
+        );
+        metadata.insert(
+            "rustscript".to_string(),
+            toml::value::Value::Table(rustscript_metadata),
+        );
+        package_map.insert("metadata".to_string(), toml::value::Value::Table(metadata));
+    }
+
+    let mut release_map = toml::map::Map::new();
+    release_map.insert("strip".to_string(), toml::value::Value::Boolean(true));
+
+    let mut profile_map = toml::map::Map::new();
+    profile_map.insert(
+        "release".to_string(),
+        toml::value::Value::Table(release_map),
+    );
+
+    let mut bin_map = toml::map::Map::new();
+    bin_map.insert(
+        "name".to_string(),
+        toml::value::Value::String(bin_name.to_string()),
+    );
+    bin_map.insert(
+        "path".to_string(),
+        toml::value::Value::String(script_name.to_string()),
+    );
+
+    let mut mani_map = toml::map::Map::new();
+    mani_map.insert(
+        "bin".to_string(),
+        toml::value::Value::Array(vec![toml::value::Value::Table(bin_map)]),
+    );
+    mani_map.insert(
+        "package".to_string(),
+        toml::value::Value::Table(package_map),
+    );
+    mani_map.insert(
+        "profile".to_string(),
+        toml::value::Value::Table(profile_map),
+    );
+
+    mani_map
 }
 
 /**
@@ -1078,7 +1173,7 @@ fn deps_manifest(deps: &[(String, String)]) -> MainResult<toml::value::Table> {
     let mut mani_str = String::new();
     mani_str.push_str("[dependencies]\n");
 
-    for &(ref name, ref ver) in deps {
+    for (name, ver) in deps {
         mani_str.push_str(name);
         mani_str.push('=');
 
@@ -1137,8 +1232,8 @@ fn merge_manifest(
     return Ok(into_t);
 
     fn as_table_mut(t: &mut toml::Value) -> Option<&mut toml::value::Table> {
-        match *t {
-            toml::Value::Table(ref mut t) => Some(t),
+        match t {
+            toml::Value::Table(t) => Some(t),
             _ => None,
         }
     }
@@ -1161,7 +1256,7 @@ fn fix_manifest_paths(mani: toml::value::Table, base: &Path) -> MainResult<toml:
 
     for path in paths {
         iterate_toml_mut_path(&mut mani, path, &mut |v| {
-            if let toml::Value::String(ref mut s) = *v {
+            if let toml::Value::String(s) = v {
                 if Path::new(s).is_relative() {
                     let p = base.join(&*s);
                     if let Some(p) = p.to_str() {
@@ -1198,12 +1293,12 @@ where
     let tail = &path[1..];
 
     if cur == "*" {
-        if let toml::Value::Table(ref mut tab) = *base {
+        if let toml::Value::Table(tab) = base {
             for (_, v) in tab {
                 iterate_toml_mut_path(v, tail, on_each)?;
             }
         }
-    } else if let toml::Value::Table(ref mut tab) = *base {
+    } else if let toml::Value::Table(tab) = base {
         if let Some(v) = tab.get_mut(cur) {
             iterate_toml_mut_path(v, tail, on_each)?;
         }
