@@ -7,18 +7,6 @@ fn test_script_explicit() {
     .unwrap()
 }
 
-#[test]
-fn test_script_features() {
-    let out = rust_script!("--features", "dont-panic", "tests/data/script-features.rs").unwrap();
-    scan!(out.stdout_output();
-        ("Keep calm and borrow check.") => ()
-    )
-    .unwrap();
-
-    let out = rust_script!("tests/data/script-features.rs").unwrap();
-    assert!(!out.success());
-}
-
 #[test]
 fn test_script_full_block() {
     let out = rust_script!("tests/data/script-full-block.rs").unwrap();
@@ -46,6 +34,15 @@ fn test_script_full_line_without_main() {
     .unwrap()
 }
 
+#[test]
+fn test_script_main_with_space() {
+    let out = rust_script!("tests/data/script-main-with-spaces.rs").unwrap();
+    scan!(out.stdout_output();
+        ("Hello, World!") => ()
+    )
+    .unwrap()
+}
+
 #[test]
 fn test_script_invalid_doc_comment() {
     let out = rust_script!("tests/data/script-invalid-doc-comment.rs").unwrap();
@@ -198,11 +195,39 @@ fn test_whitespace_before_main() {
     .unwrap()
 }
 
+#[test]
+fn test_force_rebuild() {
+    for option in ["-f", "--force"] {
+        std::env::remove_var("_RUST_SCRIPT_TEST_MESSAGE");
+
+        let script_path = "tests/data/script-using-env.rs";
+        let out = rust_script!(option, script_path).unwrap();
+        scan!(out.stdout_output();
+            ("msg = undefined") => ()
+        )
+        .unwrap();
+
+        std::env::set_var("_RUST_SCRIPT_TEST_MESSAGE", "hello");
+
+        let out = rust_script!(script_path).unwrap();
+        scan!(out.stdout_output();
+            ("msg = undefined") => ()
+        )
+        .unwrap();
+
+        let out = rust_script!(option, script_path).unwrap();
+        scan!(out.stdout_output();
+            ("msg = hello") => ()
+        )
+        .unwrap();
+    }
+}
+
 #[test]
 #[ignore]
 fn test_stable_toolchain() {
     let out = rust_script!(
-        "--toolchain-version",
+        "--toolchain",
         "stable",
         "tests/data/script-unstable-feature.rs"
     )
@@ -215,7 +240,7 @@ fn test_stable_toolchain() {
 #[ignore]
 fn test_nightly_toolchain() {
     let out = rust_script!(
-        "--toolchain-version",
+        "--toolchain",
         "nightly",
         "tests/data/script-unstable-feature.rs"
     )
@@ -235,3 +260,13 @@ fn test_same_flags() {
     )
     .unwrap()
 }
+
+#[cfg(unix)]
+#[test]
+fn test_extern_c_main() {
+    let out = rust_script!("tests/data/extern-c-main.rs").unwrap();
+    scan!(out.stdout_output();
+        ("hello, world") => ()
+    )
+    .unwrap()
+}
